/*******************************************************************************
!   Copyright (C) 2009 Intel Corporation. All Rights Reserved.
!
!   The information and  material ("Material") provided below is  owned by Intel
!   Corporation  or its  suppliers  or  licensors, and  title  to such  Material
!   remains with Intel Corporation or  its suppliers or licensors.  The Material
!   contains proprietary  information of Intel  or its suppliers  and licensors.
!   The Material is protected by worldwide copyright laws and treaty provisions.
!   No  part  of  the  Material  may  be  used,  copied,  reproduced,  modified,
!   published, uploaded,  posted, transmitted,  distributed or disclosed  in any
!   way without Intel's prior express  written permission.  No license under any
!   patent, copyright or  other intellectual property rights in  the Material is
!   granted  to  or  conferred  upon  you,  either  expressly,  by  implication,
!   inducement,  estoppel or  otherwise.   Any license  under such  intellectual
!   property rights must be express and approved by Intel in writing.
!******************************************************************************/
using System;
using System.Security;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using mkl;


/**
 * Example showing how to call the Intel MKL BLAS cgemm()
 * function using the CBLAS interface.
 */
public unsafe class test_cscal
{

    private test_cscal() {}
    public static void Main(string[] args) {

      /* Data initialization */
      int N=6, incx = 1;

      Complex[] X = new Complex[]
      {
        new Complex(1, 2), new Complex( 3,-4), new Complex( 3,3),
        new Complex(1,-7), new Complex(11,1),  new Complex(-1,2)
      };

      Complex alpha = new Complex(0,1);

    	Console.WriteLine("MKL cblas_cscal example");
  		Console.WriteLine();

      /* Print initial data */
      Console.Write(String.Format("alpha  = ({0,3}, {1,3})\n", alpha.re,  alpha.im));
      Console.WriteLine();
      printComplexVector("Vector X",X,N);

      /* Computation */
      CBLAS.cscal(N, alpha, X, incx);

      /* Print the result */
      Console.WriteLine();
      printComplexVector("Resulting X",X,N);

	}

    /** Print the vector X  */
    private static void printComplexVector(String prompt, Complex[] X, int n) {
        Console.WriteLine(prompt);
        for (int i=0; i<n; i++) {
                Console.WriteLine(String.Format("({0,3}, {1,3})",X[i].re, X[i].im));
        }
    }
}

namespace mkl
{
  [StructLayout(LayoutKind.Sequential,CharSet=CharSet.Ansi)]
  public unsafe struct Complex
  {
    public float re, im;

    public Complex(float re, float im)
    {
      this.re = re;
      this.im = im;
    }
  }

	/** CBLAS wrappers */
	public unsafe sealed class CBLAS
	{
		private CBLAS() {}

		/** CBLAS cblas_cscal wrapper */
    public static void cscal(int n,Complex a, Complex[] x, int incx)
    {
      fixed (Complex* p = &x[0])
      CBLASNative.cblas_cscal(n, &a, p, incx);
    }
	}

	/** CBLAS native declarations */
	[SuppressUnmanagedCodeSecurity]
	internal sealed unsafe class CBLASNative
	{
		private CBLASNative() {}

		/** CBLAS native cblas_cscal declaration */
    [DllImport("mkl_rt.dll", CallingConvention=CallingConvention.Cdecl,
    	 ExactSpelling=true, SetLastError=false)]
    internal static extern void cblas_cscal(
    int N, Complex* alpha , Complex* p, int incx);

  }





}
