/*******************************************************************************
!   Copyright (C) 2009 Intel Corporation. All Rights Reserved.
!
!   The information and  material ("Material") provided below is  owned by Intel
!   Corporation  or its  suppliers  or  licensors, and  title  to such  Material
!   remains with Intel Corporation or  its suppliers or licensors.  The Material
!   contains proprietary  information of Intel  or its suppliers  and licensors.
!   The Material is protected by worldwide copyright laws and treaty provisions.
!   No  part  of  the  Material  may  be  used,  copied,  reproduced,  modified,
!   published, uploaded,  posted, transmitted,  distributed or disclosed  in any
!   way without Intel's prior express  written permission.  No license under any
!   patent, copyright or  other intellectual property rights in  the Material is
!   granted  to  or  conferred  upon  you,  either  expressly,  by  implication,
!   inducement,  estoppel or  otherwise.   Any license  under such  intellectual
!   property rights must be express and approved by Intel in writing.
!******************************************************************************/
using System;
using System.Security;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using mkl;


/**
 * Example showing how to call the Intel MKL BLAS cgemm()
 * function using the CBLAS interface.
 */
public unsafe class test_zgesv
{

    private test_zgesv() {}
    public static void Main(string[] args) {

      /* Data initialization */
      int n = 4, nrhs = 2, lda = 4, ldb = 4, info;
      int[] ipiv = new int[n];
      int matrix_layout = LAPACK.ORDER.LAPACK_COL_MAJOR;
      Complex[] A = new Complex[]
      {
        new Complex( 1.23, -5.50), new Complex(-2.14, -1.12), new Complex(-4.30, -7.10), new Complex( 1.27, 7.29),
        new Complex( 7.91, -5.38), new Complex(-9.92, -0.79), new Complex(-6.47,  2.52), new Complex( 8.90, 6.92),
        new Complex(-9.80, -4.86), new Complex(-9.18, -1.12), new Complex(-6.51, -2.67), new Complex(-8.82, 1.25),
        new Complex(-7.32,  7.57), new Complex( 1.37,  0.43), new Complex(-5.86,  7.38), new Complex( 5.41, 5.37)
      };

      Complex[] B = new Complex[]
      {
        new Complex( 8.33, -7.32), new Complex(-6.18,-4.80), new Complex(-5.71,-2.80), new Complex(-1.60, 3.08),
        new Complex(-6.11, -3.81), new Complex( 0.14,-7.71), new Complex( 1.41, 3.40), new Complex( 8.54,-4.05)
      };

    	Console.WriteLine("MKL lapacke_zgesv example");
      Console.WriteLine("LAPACK_COL_MAJOR");
  		Console.WriteLine();

      /* Print initial data */
      printComplexMatrix("Matrix A",A, n, n, lda);
      printComplexMatrix("Matrix B",B, n, nrhs, ldb);

      /* Computation */
      info = LAPACK.zgesv(matrix_layout, n, nrhs, A, lda, ipiv, B, ldb);

      /* Print the results */
      if (info == 0) {
        printComplexMatrix("Solution", B, n, nrhs, ldb);
        printComplexMatrix("Details of LU factorization", A, n, n, lda);

        Console.WriteLine();
        Console.WriteLine("Pivot indices");
        for(int i = 0; i < n; i ++) {
          Console.Write(String.Format("\t {0}", ipiv[i]));
        }

      /* Bail out */
      } else {
        Console.WriteLine("The solution could not be computed.");
      }

	}

    /** Print the matrix A assuming col-major order of elements. */
    private static void printComplexMatrix(String prompt, Complex[] A, int m, int n, int lda) {
        Console.WriteLine(prompt);
        for (int i=0; i<m; i++) {
            for (int j=0; j<n; j++)
                Console.Write(String.Format("\t({0,5}, {1,5})",(A[i+j*lda].re).ToString("0.00"), (A[i+j*lda].im).ToString("0.00")));
            Console.WriteLine();
        }
    }
}

namespace mkl
{
  [StructLayout(LayoutKind.Sequential,CharSet=CharSet.Ansi)]
  public unsafe struct Complex
  {
    public double re, im;

    public Complex(double re, double im)
    {
      this.re = re;
      this.im = im;
    }
  }

	/** CBLAS wrappers */
	public unsafe sealed class LAPACK
	{
		private LAPACK() {}

    public sealed class ORDER
    {
      private ORDER() {}
      public static int LAPACK_ROW_MAJOR = 101;
      public static int LAPACK_COL_MAJOR = 102;
    }

		/** CBLAS cblas_cgemm wrapper */
		public static int zgesv(int matrix_layout, int n, int nrhs,
			Complex[] A, int lda, int[] ipiv,
			Complex[] B, int ldb)
		{
      fixed (Complex* pA = &A[0])
      fixed (Complex* pB = &B[0])
      // fixed (int* pipiv = &ipiv[0])
			return CBLASNative.LAPACKE_zgesv(matrix_layout, n, nrhs, pA, lda,
      ipiv, pB, ldb);
		}
	}

	/** CBLAS native declarations */
	[SuppressUnmanagedCodeSecurity]
	internal sealed unsafe class CBLASNative
	{
		private CBLASNative() {}

		/** CBLAS native LAPACKE_zgesv declaration */
    [DllImport("mkl_rt.dll", CallingConvention=CallingConvention.Cdecl,
    	 ExactSpelling=true, SetLastError=false)]
    internal static extern int LAPACKE_zgesv(
    int matrix_layout,
    int n,
    int nrhs,
    Complex* A,
    int lda,
    int[] ipiv,
    Complex* B,
    int ldb
    );

  }

}
